@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,7 @@
 #include "gstav.h"
 #include "gstavcodecmap.h"
 #include "gstavutils.h"
-#include "gstavpipe.h"
+#include "gstavprotocol.h"
 
 #define MAX_STREAMS 20
 
@@ -61,6 +61,9 @@ struct _GstFFMpegDemux
   /* We need to keep track of our pads, so we do so here. */
   GstPad *sinkpad;
 
+  gboolean have_group_id;
+  guint group_id;
+
   AVFormatContext *context;
   gboolean opened;
 
@@ -264,6 +267,9 @@ gst_ffmpegdemux_init (GstFFMpegDemux * demux)
   g_rec_mutex_init (&demux->task_lock);
   gst_task_set_lock (demux->task, &demux->task_lock);
 
+  demux->have_group_id = FALSE;
+  demux->group_id = G_MAXUINT;
+
   demux->opened = FALSE;
   demux->context = NULL;
 
@@ -332,7 +338,14 @@ gst_ffmpegdemux_close (GstFFMpegDemux * demux)
   demux->audiopads = 0;
 
   /* close demuxer context from ffmpeg */
-  av_close_input_file (demux->context);
+  if (demux->seekable)
+    gst_ffmpegdata_close (demux->context->pb);
+  else
+    gst_ffmpeg_pipe_close (demux->context->pb);
+  demux->context->pb = NULL;
+  avformat_close_input (&demux->context);
+  if (demux->context)
+    avformat_free_context (demux->context);
   demux->context = NULL;
 
   GST_OBJECT_LOCK (demux);
@@ -802,6 +815,24 @@ gst_ffmpegdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       res = TRUE;
       break;
     }
+    case GST_QUERY_SEGMENT:{
+      GstFormat format;
+      gint64 start, stop;
+
+      format = demux->segment.format;
+
+      start =
+          gst_segment_to_stream_time (&demux->segment, format,
+          demux->segment.start);
+      if ((stop = demux->segment.stop) == -1)
+        stop = demux->segment.duration;
+      else
+        stop = gst_segment_to_stream_time (&demux->segment, format, stop);
+
+      gst_query_set_segment (query, demux->segment.rate, format, start, stop);
+      res = TRUE;
+      break;
+    }
     default:
       /* FIXME : ADD GST_QUERY_CONVERT */
       res = gst_pad_query_default (pad, parent, query);
@@ -918,6 +949,7 @@ gst_ffmpegdemux_get_stream (GstFFMpegDemux * demux, AVStream * avstream)
   const gchar *codec;
   AVCodecContext *ctx;
   GstFFStream *stream;
+  GstEvent *event;
   gchar *stream_id;
 
   ctx = avstream->codec;
@@ -995,9 +1027,25 @@ gst_ffmpegdemux_get_stream (GstFFMpegDemux * demux, AVStream * avstream)
 
 
   stream_id =
-      gst_pad_create_stream_id_printf (pad, GST_ELEMENT_CAST (demux), "%u",
+      gst_pad_create_stream_id_printf (pad, GST_ELEMENT_CAST (demux), "%03u",
       avstream->index);
-  gst_pad_push_event (pad, gst_event_new_stream_start (stream_id));
+
+  event = gst_pad_get_sticky_event (demux->sinkpad, GST_EVENT_STREAM_START, 0);
+  if (event) {
+    if (gst_event_parse_group_id (event, &demux->group_id))
+      demux->have_group_id = TRUE;
+    else
+      demux->have_group_id = FALSE;
+    gst_event_unref (event);
+  } else if (!demux->have_group_id) {
+    demux->have_group_id = TRUE;
+    demux->group_id = gst_util_group_id_next ();
+  }
+  event = gst_event_new_stream_start (stream_id);
+  if (demux->have_group_id)
+    gst_event_set_group_id (event, demux->group_id);
+
+  gst_pad_push_event (pad, event);
   g_free (stream_id);
 
   GST_INFO_OBJECT (pad, "adding pad with caps %" GST_PTR_FORMAT, caps);
@@ -1115,9 +1163,9 @@ gst_ffmpegdemux_read_tags (GstFFMpegDemux * demux)
 static gboolean
 gst_ffmpegdemux_open (GstFFMpegDemux * demux)
 {
+  AVIOContext *iocontext = NULL;
   GstFFMpegDemuxClass *oclass =
       (GstFFMpegDemuxClass *) G_OBJECT_GET_CLASS (demux);
-  gchar *location;
   gint res, n_streams, i;
 #if 0
   /* Re-enable once converted to new AVMetaData API
@@ -1133,15 +1181,14 @@ gst_ffmpegdemux_open (GstFFMpegDemux * demux)
 
   /* open via our input protocol hack */
   if (demux->seekable)
-    location = g_strdup_printf ("gstreamer://%p", demux->sinkpad);
+    res = gst_ffmpegdata_open (demux->sinkpad, AVIO_FLAG_READ, &iocontext);
   else
-    location = g_strdup_printf ("gstpipe://%p", &demux->ffpipe);
-  GST_DEBUG_OBJECT (demux, "about to call av_open_input_file %s", location);
+    res = gst_ffmpeg_pipe_open (&demux->ffpipe, AVIO_FLAG_READ, &iocontext);
 
-  res = av_open_input_file (&demux->context, location,
-      oclass->in_plugin, 0, NULL);
+  demux->context = avformat_alloc_context ();
+  demux->context->pb = iocontext;
+  res = avformat_open_input (&demux->context, NULL, oclass->in_plugin, NULL);
 
-  g_free (location);
   GST_DEBUG_OBJECT (demux, "av_open_input returned %d", res);
   if (res < 0)
     goto open_failed;
@@ -1377,7 +1424,7 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
   srcpad = stream->pad;
 
   rawvideo = (avstream->codec->codec_type == AVMEDIA_TYPE_VIDEO &&
-      avstream->codec->codec_id == CODEC_ID_RAWVIDEO);
+      avstream->codec->codec_id == AV_CODEC_ID_RAWVIDEO);
 
   if (rawvideo)
     outsize = gst_ffmpeg_avpicture_get_size (avstream->codec->pix_fmt,
@@ -1848,6 +1895,8 @@ gst_ffmpegdemux_change_state (GstElement * element, GstStateChange transition)
           NULL);
       g_list_free (demux->cached_events);
       demux->cached_events = NULL;
+      demux->have_group_id = FALSE;
+      demux->group_id = G_MAXUINT;
       break;
     default:
       break;
